package rule

import (
	"code.fnuoos.com/go_rely_warehouse/zyos_go_order_relate_rule.git/db"
	"code.fnuoos.com/go_rely_warehouse/zyos_go_order_relate_rule.git/db/model"
	"code.fnuoos.com/go_rely_warehouse/zyos_go_order_relate_rule.git/md"
	zhios_order_relate_utils "code.fnuoos.com/go_rely_warehouse/zyos_go_order_relate_rule.git/utils"
	zhios_order_relate_logx "code.fnuoos.com/go_rely_warehouse/zyos_go_order_relate_rule.git/utils/logx"
	"encoding/json"
	"errors"
	"fmt"
	"math"
	"strconv"
	"strings"
	"time"
	"xorm.io/xorm"
)

// BatchGetPublicPlatoonRelateCommission 批量分佣 订单显示
func BatchGetPublicPlatoonRelateCommissionByOrder(engine *xorm.Engine, masterId string, PublicPlatoonRelateCommissionReqList []*md.PublicPlatoonRelateCommissionReq) (map[string]*md.PublicPlatoonRelateCommissionResp, error) {
	var resp map[string]*md.PublicPlatoonRelateCommissionResp
	userPublicPlatoonSetting, err := db.UserPublicPlatoonSettingGetOneByParams(engine, map[string]interface{}{
		"key":   "is_open",
		"value": 1,
	})
	if err != nil {
		return nil, err
	}
	for _, param := range PublicPlatoonRelateCommissionReqList {
		// 根据供应商
		rewardOpt, err := db.DbsPlanRewardByPvd(engine, param.Pvd)
		if err != nil {
			return nil, err
		}
		if rewardOpt == nil {
			return nil, zhios_order_relate_logx.Warn("找不到方案记录")
		}
		if rewardOpt.State == 0 {
			return nil, zhios_order_relate_logx.Warn("抽成方案未开启")
		}
		if rewardOpt.PlanCommissionId == 0 {
			return nil, zhios_order_relate_logx.Warn("抽成方案未设置佣金方案id")
		}
		fmt.Println("抽成设置:", zhios_order_relate_utils.SerializeStr(rewardOpt))
		fmt.Println("commission id:", rewardOpt.PlanCommissionId)

		//TODO::计算共富收益
		commissionOpt, err := db.DbsPlanCommissionById(engine, rewardOpt.PlanCommissionId)
		if err != nil || commissionOpt == nil || commissionOpt.Id == 0 {
			return nil, err
		}
		if err != nil || commissionOpt == nil || commissionOpt.Id == 0 {
			return nil, err
		}
		var subsidyTmp map[string]*md.LvGrade
		if err := json.Unmarshal([]byte(commissionOpt.Data), &subsidyTmp); err != nil {
			return nil, zhios_order_relate_logx.Warn(fmt.Sprintf("%s:分佣方案数据设置错误", masterId))
		}
		lvGrade := subsidyTmp[param.UserLevel]
		var selfRateList, commonWealthSystem, directPush map[string]string
		for _, coinId := range lvGrade.ReturnType {
			selfRateList[coinId] = zhios_order_relate_utils.Float64ToStr(zhios_order_relate_utils.StrToFloat64(lvGrade.SelfRateList[coinId]) * zhios_order_relate_utils.StrToFloat64(param.PendingAmount))
			commonWealthSystem[coinId] = zhios_order_relate_utils.Float64ToStr(zhios_order_relate_utils.StrToFloat64(lvGrade.CommonWealthSystem[coinId]) * zhios_order_relate_utils.StrToFloat64(param.PendingAmount))
			directPush[coinId] = zhios_order_relate_utils.Float64ToStr(zhios_order_relate_utils.StrToFloat64(lvGrade.DirectPush[coinId]) * zhios_order_relate_utils.StrToFloat64(param.PendingAmount))
		}

		//TODO::本次消费产生的共富收益给到谁
		var nowBenefitUid string
		userPublicPlatoonRelation, err := db.UserPublicPlatoonRelationGetOneByParams(engine, map[string]interface{}{
			"key":   "uid",
			"value": param.Uid,
		})
		if err != nil {
			return nil, err
		}
		if userPublicPlatoonRelation == nil {
			return nil, errors.New("未查询到公排关系记录")
		}
		fatherUids := strings.Split(userPublicPlatoonRelation.FatherUid, "-")
		userPublicPlatoonRelation.ReturnCommissionNum++
		remainder := (userPublicPlatoonRelation.ReturnCommissionNum) % userPublicPlatoonSetting.SeveralRows
		if remainder == 0 {
			nowBenefitUid = fatherUids[userPublicPlatoonSetting.SeveralRows-1]
		} else {
			nowBenefitUid = fatherUids[remainder-1]
		}

		//TODO::更新 公排关系记录 中 `return_commission_num`
		affected, err := db.UserPublicPlatoonRelationUpdate(engine, userPublicPlatoonRelation.Id, userPublicPlatoonRelation, "return_commission_num")
		if err != nil {
			return nil, err
		}
		if affected == 0 {
			return nil, errors.New("更新公排关系记录失败")
		}
		resp[param.Oid] = &md.PublicPlatoonRelateCommissionResp{
			Uid:                    param.Uid,
			CommonWealthBenefitUid: nowBenefitUid,
			DirectPushBenefitUid:   strconv.Itoa(userPublicPlatoonRelation.RecommendUid),
			PendingAmount:          param.PendingAmount,
			Oid:                    param.Oid,
			SelfRateList:           selfRateList,
			CommonWealthSystem:     commonWealthSystem,
			DirectPush:             directPush,
		}
	}
	return resp, nil
}

// AddPublicPlatoonRelateCommission 新增公排用户关系记录
func AddPublicPlatoonRelateCommission(engine *xorm.Engine, masterId string, AddPublicPlatoonRelateCommissionReqList []*md.AddPublicPlatoonRelateCommissionReq) (map[string]*model.UserPublicPlatoonRelation, error) {
	var resp map[string]*model.UserPublicPlatoonRelation
	//查找 `user_public_platoon_setting` 基础设置
	userPublicPlatoonSetting, err := db.UserPublicPlatoonSettingGetOneByParams(engine, map[string]interface{}{
		"key":   "is_open",
		"value": 1,
	})
	if err != nil {
		return nil, err
	}
	for _, param := range AddPublicPlatoonRelateCommissionReqList {
		res, err := publicPlatoon(engine, zhios_order_relate_utils.StrToInt(param.Uid), zhios_order_relate_utils.StrToInt(param.RecommendUid), *userPublicPlatoonSetting)
		if err != nil {
			return nil, err
		}
		resp[param.Uid] = &res
	}
	return resp, nil
}

/*
  publicPlatoon 公排方法
  TODO 相关公式:
  1: 每个等级的起始值(1+5^0+5^1+5^2+...+5^x-2), 每个等级的结束值(`5^0+5^1+5^2+...+5^x-1)
  2: 根据position查找pid { (position-1)/5 }
  3: 根据position查找等级level {position-5^0-5^1-5^2-...-5^x 是否 <0 ?  => x+1 }
*/
func publicPlatoon(engine *xorm.Engine, uid, recommendUid int, userPublicPlatoonSetting model.UserPublicPlatoonSetting) (model.UserPublicPlatoonRelation, error) {
	var userPublicPlatoonRelation model.UserPublicPlatoonRelation
	//1、 查找当前 user_public_platoon_relation 中 `position` 最大的记录
	var m model.UserPublicPlatoonRelation
	if has, err := engine.OrderBy("position desc").Get(&m); err != nil || has == false {
		return userPublicPlatoonRelation, err
	}

	position := m.Position + 1
	pid := makeSearchPid(position, userPublicPlatoonSetting.SeveralTimes)
	var level float64
	makeSearchLevel(&position, float64(userPublicPlatoonSetting.SeveralTimes), &level)
	level = level + 1 //TODO::根据公式需要 + 1

	//2、查找对应pid的 user_public_platoon_relation 记录
	var m1 model.UserPublicPlatoonRelation
	if has, err := engine.Where("uid=?", pid).Get(&m1); err != nil || has == false {
		return userPublicPlatoonRelation, err
	}
	var fatherUid string
	if m1.FatherUid == "" {
		//TODO::顶级
		fatherUid = zhios_order_relate_utils.IntToStr(m1.Uid)
	} else {
		fatherUid = zhios_order_relate_utils.IntToStr(m1.Uid) + "-" + m1.FatherUid
	}

	//唯一标识符(父级id-uid-等级-位置)
	var uniqueIdentifier = zhios_order_relate_utils.IntToStr(pid) + zhios_order_relate_utils.IntToStr(uid) + zhios_order_relate_utils.Float64ToStr(level) + zhios_order_relate_utils.IntToStr(position)

	// 3、插入 user_public_platoon_relation 记录
	now := time.Now()
	userPublicPlatoonRelation.Uid = uid
	userPublicPlatoonRelation.FatherUid = fatherUid
	userPublicPlatoonRelation.Pid = pid
	userPublicPlatoonRelation.RecommendUid = recommendUid
	userPublicPlatoonRelation.Level = int(level)
	userPublicPlatoonRelation.Position = position
	userPublicPlatoonRelation.UniqueIdentifier = uniqueIdentifier
	userPublicPlatoonRelation.CreateAt = now
	userPublicPlatoonRelation.UpdateAt = now
	_, err := db.UserPublicPlatoonRelationInsert(engine, &userPublicPlatoonRelation)
	if err != nil {
		return model.UserPublicPlatoonRelation{}, err
	}
	return userPublicPlatoonRelation, nil
}

//递归查找等级
func makeSearchLevel(position *int, rows float64, times *float64) (level int) {
	difference := *position - int(math.Pow(rows, *times))
	if difference <= 0 {
		return int(*times)
	} else {
		position = &difference
		*times++
		return makeSearchLevel(position, rows, times)
	}
}

//查找归属父级id
func makeSearchPid(position int, row int) (pid int) {
	divisionValue := (position - 1) / row
	if divisionValue == 0 {
		pid = 1
		return
	} else {
		if (divisionValue*row + 1) == position {
			pid = divisionValue
			return
		} else {
			pid = divisionValue + 1
			return
		}
	}
}
